// http://graphviz.org
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

public class DotMakerGraph {
  private final Graph graph;
  private final StringBuilder b = new StringBuilder();
  private final Map<Vertex, Integer> identifiers = new HashMap<Vertex, Integer>();
  private int nextId = 0;

  public static String makeDot(Graph graph) {
    return new DotMakerGraph(graph).getDot();
  }

  private DotMakerGraph(Graph graph) {
    this.graph = graph;
  }

  private String getDot() {
    b.append("digraph MyGraph {\n");
    for (Object v: graph.vertices) {
      vertex((Vertex) v);
    }
    for (Object e: graph.edges) {
      edge((Edge) e);
    }
    b.append("}");
    return b.toString();
  }

  private void vertex(Vertex v) {
    createIdentifier(v);
    b.append(String.format("node%03d[label=\"%s\"]", 
      identifiers.get(v), v.value) + ";\n");
  }

  private void createIdentifier(Vertex v) {
    if (!identifiers.containsKey(v)) {
      identifiers.put(v, ++nextId);
    }
  }

  private void edge(Edge e) {
    createIdentifier(e.start);
    createIdentifier(e.end);
    b.append(String.format("node%03d", identifiers.get(e.start)));
    b.append("->"); 
    b.append(String.format("node%03d", identifiers.get(e.end)));
    b.append("[");
    if (e.weight != 0) {
      b.append(String.format("label=\"%d\",", e.weight));
    }
    switch(e.kind) {
      case Directed:
        b.append("dir=forward,");
        break;
      case Undirected:
        b.append("dir=none,");
        break;
      default:
        throw new UnsupportedOperationException();
    }
    b.append("]");
    b.append(";\n");
  }
}